package jp.haw.grain.transform;

import java.util.List;

import jp.haw.grain.transform.transformer.AbstractTransformer;
import jp.haw.grain.transform.transformer.Html2Form;
import jp.haw.grain.transform.util.NamespaceMap;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * ϊۂɂȂNXłB
 * @author nakajo
 */
public class TransformOperation {

	protected Document gudDoc;
	protected Element current;
	protected NamespaceMap rootNamespaceMap;
	protected AbstractTransformer transformer;
	protected int deep;

	/**
	 * RXgN^
	 * ݂̃Jgm[hw肵āA<code>TransformeOperation</code>쐬܂B
	 * ̂Ƃgud\z邽߂DocumentnodeownerDocumentɂȂ܂B
	 * @param doc
	 * @param node
	 */
	public TransformOperation(Element element) {
		this.gudDoc = element.getOwnerDocument();
		this.current = element;
		this.deep = 0;

		this.rootNamespaceMap = new NamespaceMap(this.gudDoc.createElement("mapping_root"));
		this.rootNamespaceMap.addUriPrefix(GudBuilder.GUD_NS, NamespaceMap.EMPTY_PREFIX);
	}

	public TransformOperation(Element element, NamespaceMap mapping, int deep) {
		this.gudDoc = element.getOwnerDocument();
		this.current = element;

		this.deep = deep;
		this.rootNamespaceMap = mapping;
	}

	public void operation(AbstractTransformer t) {
		this.transformer = t;
		DocumentFragment df = t.transform();

		//OI html2form̂Ƃ̓[gGg邪nullԂ̂
		//̂Ƃ͗OBBBB
		if (t instanceof Html2Form) {
			registNamespaceMapping(this.gudDoc.getDocumentElement(), this.rootNamespaceMap);
		}

		NamespaceMap map = searchMapping(this.rootNamespaceMap, this.current);

		if (df != null) {
			//dfvf݂͌current̎qɂȂ̂ŁAnamespaceMapping݂currentowner̂
			//ł΃JgmappingƂȂB
			int start = this.current.getChildNodes().getLength();
			this.current.appendChild(df);

			NodeList list = this.current.getChildNodes();
			int end = list.getLength();
			for (int i = start; i < end; i++) {
				Node node = list.item(i);
				registNamespaceMapping(node, map);
				resolvePrefix(node);
			}
		}

	}

	protected void registNamespaceMapping(Node node, NamespaceMap currentNMap) {
		if (node.getNodeType() == Node.TEXT_NODE)
			return;
		if (currentNMap == null)
			throw new RuntimeException("Namespace can not resolve: deep=" + this.deep + " node=" + node.getClass().getName() + "[" + node.getNodeName() + ": " + node.getNamespaceURI() + ": " + node.getNodeValue());

		NamespaceMap nMap = new NamespaceMap(node);

		if (node.hasAttributes()) {
			NamedNodeMap attrs = node.getAttributes();
			for (int i = 0; i < attrs.getLength(); i++) {
				Node attr = attrs.item(i);
				if (!attr.getNodeName().startsWith("xmlns"))
					continue;

				String uri = attr.getNodeValue();
				String prefix = null;
				if (attr.getNodeName().indexOf(":") == -1) {
					//prefixȂB
					prefix = NamespaceMap.EMPTY_PREFIX;
				} else {
					String name = attr.getNodeName();
					prefix = name.substring(name.indexOf(":") + 1);
				}

				//System.out.println("["+node.getNodeName()+"]"+"uri="+uri+" :prefix="+prefix);
				nMap.addUriPrefix(uri, prefix);
			}
		}

		currentNMap.addChild(nMap);

		if (!node.hasChildNodes())
			return;

		NodeList list = node.getChildNodes();
		for (int i = 0; i < list.getLength(); i++) {
			registNamespaceMapping(list.item(i), nMap);
		}
	}

	protected NamespaceMap searchMapping(NamespaceMap mapping, Node node) {
		NamespaceMap searchMapping = null;

		if (mapping.getOwnerNode().equals(node)) {
			return mapping;
		}

		if (!mapping.hasChildren())
			return null;
		List children = mapping.getChildren();

		for (int i = 0; i < children.size(); i++) {
			NamespaceMap child = (NamespaceMap)children.get(i);
			searchMapping = searchMapping(child, node);
		}

		return searchMapping;
	}

	protected void resolvePrefix(Node node) {
		if (node.getNodeType() == Node.TEXT_NODE)
			return;

		String uri = node.getNamespaceURI();
		if (uri == null)
			uri = NamespaceMap.EMPTY_URI;

		String prefix = resolvePrefix(uri, searchMapping(this.rootNamespaceMap, node));
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			if (prefix != null && !prefix.equals(node.getPrefix())) {
				node.setPrefix(prefix);
			} else if (node.getPrefix() != null && node.getPrefix().equals(prefix)) {
				node.setPrefix(prefix);
			}
		}

		if (!node.hasChildNodes())
			return;
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			resolvePrefix(children.item(i));
		}
	}

	protected String resolvePrefix(String uri, NamespaceMap mapping) {
		//uriɑΉprefixTB
		//݂mappingɑ݂ȂΐeɕɂB
		if (mapping == null)
			return null;

		if (mapping.contains(uri)) {
			return mapping.getPrefix(uri);
		}

		if (mapping.getParent() == null)
			return null;

		return resolvePrefix(uri, mapping.getParent());
	}

	/**
	 * ݂̗vf̎qvfǉ邽߂̓K؂
	 * <code>TransformeOperation</code>𐶐܂B
	 * @return
	 */
	public TransformOperation createChildNodeOperation() {
		//ĂȂΎgƓcurrentoperationԂB
		if (this.transformer == null)
			return new TransformOperation(this.current, this.rootNamespaceMap, this.deep);

		Element child = this.transformer.getCurrentElement();
		if (child == null)
			return new TransformOperation(this.current, this.rootNamespaceMap, this.deep);

		return new TransformOperation(child, this.rootNamespaceMap, this.deep + 1);
	}

	public Node getCurrentNode() {
		return this.current;
	}

	public String toString() {
		return '@' + Integer.toHexString(hashCode()) + "[" + this.current.getNodeName() + "]";
	}

	public NamespaceMap getRootMap() {
		return this.rootNamespaceMap;
	}
}
