package jp.haw.grain.transform.xpath;

import jp.haw.grain.xpath.XPathByteCodeElement;
import jp.haw.grain.xpath.compiler.ASTAbbreviatedStep;
import jp.haw.grain.xpath.compiler.ASTAbsoluteLocationPath;
import jp.haw.grain.xpath.compiler.ASTAndExpr;
import jp.haw.grain.xpath.compiler.ASTAxisSpecifier;
import jp.haw.grain.xpath.compiler.ASTDivExpr;
import jp.haw.grain.xpath.compiler.ASTEqualExpr;
import jp.haw.grain.xpath.compiler.ASTFunctionCall;
import jp.haw.grain.xpath.compiler.ASTFunctionExpr;
import jp.haw.grain.xpath.compiler.ASTFunctionName;
import jp.haw.grain.xpath.compiler.ASTGeExpr;
import jp.haw.grain.xpath.compiler.ASTGtExpr;
import jp.haw.grain.xpath.compiler.ASTLeExpr;
import jp.haw.grain.xpath.compiler.ASTLiteral;
import jp.haw.grain.xpath.compiler.ASTLtExpr;
import jp.haw.grain.xpath.compiler.ASTMinusExpr;
import jp.haw.grain.xpath.compiler.ASTModExpr;
import jp.haw.grain.xpath.compiler.ASTMulExpr;
import jp.haw.grain.xpath.compiler.ASTNegExpr;
import jp.haw.grain.xpath.compiler.ASTNodePI;
import jp.haw.grain.xpath.compiler.ASTNodeType;
import jp.haw.grain.xpath.compiler.ASTNotEqualExpr;
import jp.haw.grain.xpath.compiler.ASTNumber;
import jp.haw.grain.xpath.compiler.ASTOrExpr;
import jp.haw.grain.xpath.compiler.ASTPlusExpr;
import jp.haw.grain.xpath.compiler.ASTPredicate;
import jp.haw.grain.xpath.compiler.ASTQName;
import jp.haw.grain.xpath.compiler.ASTRelativeLocationPath;
import jp.haw.grain.xpath.compiler.ASTSlashExpr;
import jp.haw.grain.xpath.compiler.ASTStep;
import jp.haw.grain.xpath.compiler.ASTUnionExpr;
import jp.haw.grain.xpath.compiler.ASTVariableReference;
import jp.haw.grain.xpath.compiler.ASTXPathExpression;
import jp.haw.grain.xpath.compiler.SimpleNode;
import jp.haw.grain.xpath.compiler.XPathParserConstants;
import jp.haw.grain.xpath.compiler.XPathParserTreeConstants;
import jp.haw.grain.xpath.compiler.XPathParserVisitor;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class GudXPathVisitor implements XPathParserVisitor {

	/** XPathoCgR[hpm[hDocumentIuWFNg */
	protected Document ownerDoc;

	public GudXPathVisitor(Document doc) {
		this.ownerDoc = doc;
	}

	protected Node rootElement(Node e) {
		Node parent = e.getParentNode();
		if (parent == null) return e;

		return rootElement(parent);
	}

	public Object visit(ASTXPathExpression node, Object data) {
		if (node.jjtGetNumChildren() > 1) throw new RuntimeException("to many children!");
		Element element = (Element) node.jjtGetChild(0).jjtAccept(this, data);

		return rootElement(element);
	}

	public Object visit(ASTAbsoluteLocationPath node, Object data) {
		// qvf͕KP̂͂BBBB
		if (node.jjtGetNumChildren() > 1) throw new RuntimeException("to many children!");

		XPathByteCodeElement absoluteAxis = XPathByteCodeElement.createAxisElement("absolute", this.ownerDoc);
		XPathByteCodeElement ctxn = XPathByteCodeElement.createContextLoadElement(this.ownerDoc);
		XPathByteCodeElement step = XPathByteCodeElement.createStepElement(this.ownerDoc);

		step.setAttribute("_0", "");
		step.setAttribute("_1", "");
		step.appendChild(absoluteAxis);
		step.appendChild(ctxn);

		Element child = null;
		if(node.jjtGetNumChildren() != 0) child = (Element) node.jjtGetChild(0).jjtAccept(this, data);
		if (node.getToken().kind == XPathParserConstants.SLASHSLASH) {
			Element slashSlash = slashSlashExpr(data);
			slashSlash.appendChild(step);

			if(child != null) child.appendChild(slashSlash);

		} else {
			if(child != null) child.appendChild(step);
		}
		return ctxn;
	}

	public Object visit(ASTRelativeLocationPath node, Object data) {
		// qvf͕KP̂͂BBBB
		if (node.jjtGetNumChildren() > 1) throw new RuntimeException("to many children!");

		Element child = (Element) node.jjtGetChild(0).jjtAccept(this, data);
		XPathByteCodeElement ctxn = XPathByteCodeElement.createContextLoadElement(this.ownerDoc);
		child.appendChild(ctxn);

		return child;
	}

	public Object visit(ASTAbbreviatedStep node, Object data) {
		Element step = XPathByteCodeElement.createStepElement(this.ownerDoc);
		Element axis = XPathByteCodeElement.createAxisElement(XPathByteCodeElement.AXIS_SELF, this.ownerDoc);
		
		// .@̂ƂftHgƂB
		// . => self::node() ƂȂB
		
		if(node.getToken().kind == XPathParserConstants.DOTDOT) {
			// parent::node()/ ɂȂ
			axis = XPathByteCodeElement.createAxisElement(XPathByteCodeElement.AXIS_PARENT, this.ownerDoc);
		}

		step.setAttribute("_1", "node");
		step.appendChild(axis);
		return step;
	}

	public Object visit(ASTSlashExpr node, Object data) {
		// SlashExpr͕KQqm[h
		// łׂ̏͂Ẵm[h𔽓]
		// ܂
		// 1.E(2Ԗ)̎qm[h擾
		// 2.(1Ԗ)̎qm[h1̎qm[hɃZbg
		// 3.2̃m[h^[
		if (node.jjtGetNumChildren() < 2) throw new RuntimeException("not enogh children.");
		Element rightNode = (Element) node.jjtGetChild(1).jjtAccept(this, data);
		Element leftNode = (Element) node.jjtGetChild(0).jjtAccept(this, data);

		// Slash"//"̂Ƃ͂ꎩ̂StepNodeɂȂB
		// a//b = a/dependencd-or-self::node()/b
		// ̏ꍇ̃c[
		// <step name="b"><step type="node"><step name="a" /></step></step>
		// ɂȂ͂BBB
		// ܂A right <- // <- leftappend邱ƂɂȂB
		if (node.getToken().kind == XPathParserConstants.SLASHSLASH) {
			Element slashSlash = slashSlashExpr(data);
			rightNode.appendChild(slashSlash);
			slashSlash.appendChild(leftNode);

			return leftNode;
		}

		rightNode.appendChild(leftNode);
		return leftNode;
	}

	public Object visit(ASTStep node, Object data) {

		// ̃m[h͕KqQ
		// PڂAxisQڂNodeTestAQName
		// Rڈȍ~Ƃ炷ׂpredicate
		if (node.jjtGetNumChildren() < 2) throw new RuntimeException("not enogh children.");
		Element axis = (Element) node.jjtGetChild(0).jjtAccept(this, data);
		Element step = (Element) node.jjtGetChild(1).jjtAccept(this, data);

		// Rڈȍ~BȂΉɏI
		Element previousPredicate = null;
		Element topPredicate = null;
		for (int i = 2; i < node.jjtGetNumChildren(); i++) {
			Element predicate = (Element) node.jjtGetChild(i).jjtAccept(this, data);
			if (previousPredicate != null) {
				predicate.appendChild(previousPredicate);
			}

			// 擪predicateoĂB
			// step̎qƂĒǉ邽߁B
			if (topPredicate == null) topPredicate = predicate;

			previousPredicate = predicate;
		}
		if (axis != null) {
			step.appendChild(axis);
		}
		if (topPredicate != null) {
			topPredicate.appendChild(step);
		}
		return step;
	}

	public Object visit(ASTAxisSpecifier node, Object data) {
		// Axis
		// Tokenۂ̂Ƃ̓ftHgȂ̂childɂB
		// @̂ƂattributeɂȂB

		if (node.getToken() == null) { // defaultchild
			return XPathByteCodeElement.createAxisElement("child", this.ownerDoc);
		} else if (node.getToken().kind == XPathParserConstants.ABBREVIATED_ATTRIBUTE_AXIS) {
			return XPathByteCodeElement.createAxisElement("attribute", this.ownerDoc);
		}

		return XPathByteCodeElement.createAxisElement(node.getToken().image, this.ownerDoc);
	}

	public Object visit(ASTNodeType node, Object data) {
		// QNamestepm[hƂȂB
		// mɂstepm[htypeB
		// Ȃ̂łstepm[h𐶐

		Element step = XPathByteCodeElement.createStepElement(this.ownerDoc);
		step.setAttribute("_1", node.getToken().image);
		step.setAttribute("_0", "");

		return step;
	}

	public Object visit(ASTNodePI node, Object data) {
		// TODO ꂽ\bhEX^u
		return null;
	}

	public Object visit(ASTQName node, Object data) {
		// QNamestepm[hƂȂB
		// mɂstepm[hnameB
		// Ȃ̂łstepm[h𐶐

		Element step = XPathByteCodeElement.createStepElement(this.ownerDoc);
		String prefix = "";
		if (node.getPrefix() != null) {
			prefix = node.getPrefix().image;
		}

		//step.setAttribute("_2", prefix);
		step.setAttribute("_1", "");
		if(node.getLocal().kind != XPathParserConstants.STAR) {
			step.setAttribute("_0", node.getLocal().image);
		}
		
		return step;
	}

	public Object visit(ASTPredicate node, Object data) {
		// Predicate
		// <filter><pushclosure>`</pushclosure></filter>ƂȂ
		// Ȃ̂ŁA悸͎qvfɕϊ
		// ̌Aϊꂽqvfg̎qvfɒǉ

		Element filter = XPathByteCodeElement.createFilterElement(this.ownerDoc);
		Element pushclosure = XPathByteCodeElement.createPredicateElement(this.ownerDoc);

		// qvfExpr()Ȃ̂ŕKP̂͂BBBB
		if (node.jjtGetNumChildren() > 1) throw new RuntimeException("to many children!");

		Element children = null;
		if (((SimpleNode) node.jjtGetChild(0)).id == XPathParserTreeConstants.JJTNUMBER) {
			// qNumberłposition()=Numbeř`ɓWJB
			// Number͎qvfȂ̂ŁANumber݂̂Ƃ߂ĂOK
			Element equal = XPathByteCodeElement.createEqElement(this.ownerDoc);
			Element num = (Element) node.jjtGetChild(0).jjtAccept(this, data);
			Element position = XPathByteCodeElement.createContextParamLoadElement(XPathByteCodeElement.CONTEXT_PARAM_POSITION, this.ownerDoc);
			
			equal.appendChild(position);
			equal.appendChild(num);
			
			children = equal;
		} else {
			children = (Element) node.jjtGetChild(0).jjtAccept(this, data);
		}
		// c[\\z
		filter.appendChild(pushclosure);
		pushclosure.appendChild(children);

		return filter;
	}

	public Object visit(ASTFunctionCall node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		// PԖڂ֐
		// QԖڂ
		String funcName = (String) node.jjtGetChild(0).jjtAccept(this, data);
		if (funcName.equals("position")) {
			Element ctxpl = XPathByteCodeElement.createContextParamLoadElement(XPathByteCodeElement.CONTEXT_PARAM_POSITION, this.ownerDoc);
			return ctxpl;
		} else if (funcName.equals("last")) {
			Element ctxpl = XPathByteCodeElement.createContextParamLoadElement(XPathByteCodeElement.CONTEXT_PARAM_SIZE, this.ownerDoc);
			return ctxpl;
		}

		Element fcall = XPathByteCodeElement.createFcallElement(this.ownerDoc);
		fcall.setAttribute("_0", funcName);

		SimpleNode funcExpr = (SimpleNode) node.jjtGetChild(1);
		for (int i = 0; i < funcExpr.jjtGetNumChildren(); i++) {
			Element expr = (Element) funcExpr.jjtGetChild(i).jjtAccept(this, data);
			fcall.appendChild(expr);
		}

		return fcall;
	}

	public Object visit(ASTFunctionName node, Object data) {
		String name = "";
		if (node.getPrefix() != null) {
			name = node.getPrefix().image + ":";
		}

		return name + node.getLocal().image;
	}

	public Object visit(ASTFunctionExpr node, Object data) {
		// TODO ꂽ\bhEX^u
		return null;
	}

	public Object visit(ASTAndExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement and = XPathByteCodeElement.createAndElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		and.appendChild(leftNode);
		and.appendChild(rightNode);

		return and;
	}

	public Object visit(ASTDivExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement div = XPathByteCodeElement.createDivElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		div.appendChild(leftNode);
		div.appendChild(rightNode);

		return div;
	}

	public Object visit(ASTEqualExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement equal = XPathByteCodeElement.createEqElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		equal.appendChild(leftNode);
		equal.appendChild(rightNode);

		return equal;
	}

	public Object visit(ASTGeExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement ge = XPathByteCodeElement.createGeElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		ge.appendChild(leftNode);
		ge.appendChild(rightNode);

		return ge;
	}

	public Object visit(ASTGtExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement gt = XPathByteCodeElement.createGtElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		gt.appendChild(leftNode);
		gt.appendChild(rightNode);

		return gt;
	}

	public Object visit(ASTLeExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement le = XPathByteCodeElement.createLeElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		le.appendChild(leftNode);
		le.appendChild(rightNode);

		return le;
	}

	public Object visit(ASTLtExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement lt = XPathByteCodeElement.createLtElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		lt.appendChild(leftNode);
		lt.appendChild(rightNode);

		return lt;
	}

	public Object visit(ASTMinusExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement sub = XPathByteCodeElement.createSubElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		sub.appendChild(leftNode);
		sub.appendChild(rightNode);

		return sub;
	}

	public Object visit(ASTMulExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement mult = XPathByteCodeElement.createMultElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		mult.appendChild(leftNode);
		mult.appendChild(rightNode);

		return mult;
	}

	public Object visit(ASTModExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement mod = XPathByteCodeElement.createModElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		mod.appendChild(leftNode);
		mod.appendChild(rightNode);

		return mod;
	}

	public Object visit(ASTNotEqualExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement notEqual = XPathByteCodeElement.createNeElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		notEqual.appendChild(leftNode);
		notEqual.appendChild(rightNode);

		return notEqual;
	}

	public Object visit(ASTOrExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement or = XPathByteCodeElement.createOrElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		or.appendChild(leftNode);
		or.appendChild(rightNode);

		return or;
	}

	public Object visit(ASTPlusExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement add = XPathByteCodeElement.createAddElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		add.appendChild(leftNode);
		add.appendChild(rightNode);

		return add;
	}

	public Object visit(ASTUnionExpr node, Object data) {
		// ̃m[h͕KqQ
		if (node.jjtGetNumChildren() != 2) throw new RuntimeException("not enogh children.");

		XPathByteCodeElement union = XPathByteCodeElement.createUnionElement(this.ownerDoc);
		Element leftNode = (Element) rootElement((Node) node.jjtGetChild(0).jjtAccept(this, data));
		Element rightNode = (Element) rootElement((Node) node.jjtGetChild(1).jjtAccept(this, data));

		union.appendChild(leftNode);
		union.appendChild(rightNode);

		return union;
	}

	public Object visit(ASTLiteral node, Object data) {
		// vfɕϊ
		return XPathByteCodeElement.createStringValueElement(node.getToken().image, this.ownerDoc);
	}

	public Object visit(ASTNumber node, Object data) {
		// vfɕϊ
		return XPathByteCodeElement.createNumValueElement(Integer.parseInt(node.getToken().image), this.ownerDoc);
	}

	public Object visit(ASTNegExpr node, Object data) {
		// ̃m[h͕Kq1
		if (node.jjtGetNumChildren() != 1) throw new RuntimeException("not enogh children.");

		SimpleNode child = (SimpleNode) node.jjtGetChild(0);
		int count = 1;
		while (child.id == XPathParserTreeConstants.JJTNEGEXPR) {
			count++;
			child = (SimpleNode) child.jjtGetChild(0);
		}

		if ((count % 2) == 1) {
			Element neg = XPathByteCodeElement.createNegElement(this.ownerDoc);
			Element childExpr = (Element) child.jjtAccept(this, data);

			neg.appendChild(childExpr);
			return neg;
		}

		return child.jjtAccept(this, data);
	}

	public Object visit(ASTVariableReference node, Object data) {
		return null;
	}

	public Object visit(SimpleNode node, Object data) {
		return null;
	}

	protected Element slashSlashExpr(Object data) {
		XPathByteCodeElement step = XPathByteCodeElement.createStepElement(this.ownerDoc);
		XPathByteCodeElement axis = XPathByteCodeElement.createAxisElement(XPathByteCodeElement.AXIS_DESCENDANT_OR_SELF, this.ownerDoc);
		step.setAttribute("type", "node");
		step.appendChild(axis);

		return step;
	}

	protected void showNode(SimpleNode node) {

		System.out.print(node.toString(""));
		if (node.getToken() != null) {
			System.out.println("[token: " + node.getToken().image + "]");
		} else {
			System.out.println("");
		}
		for (int i = 0; i < node.jjtGetNumChildren(); i++) {
			node.jjtGetChild(i).jjtAccept(this, null);
		}

	}

}
